﻿-- 
-- MultiPlaceableTypes - This Script allows you to have more then one 'placeableType' in your placeable Mod.
-- 
-- @Interface: 1.4.4.0 1.4.4RC8
-- @Author: kevink98 
-- @Date: 20.07.2017
-- @Version: 1.1
-- 
-- @Support: http://ls-modcompany.com
-- @mCompanyInfo: http://mcompany-info.de/
-- 

--[[Instructions:
	<placeableType> (class of Mainmod) </placeableType>   <!-- this is from Giants-->
    
	<!-- for this script:-->	
	<multiPlaceableTypes>
		<placeableType class="animatedObjectPlaceable" xmlFilename="animatedObjects.xml"/>
	</multiPlaceableTypes>

]]--

--[[
	## V0.2.1 (12.05.2017) Open Beta:
			- first official Release (Beta)
	## V1.0 (16.06.2017):
			- Fix Error, if Mod delete
	## V1.1 (11.07.2017):
			- Fix Errors: 
				- if close Map
				- if delete placeable Mod
			- Support now more Mods whith this script as one (Thank to GtX)
]]--

MultiPlaceableTypes = {};
MultiPlaceableTypes.ModDir = g_currentModDirectory
MultiPlaceableTypes.storeItems = {};
MultiPlaceableTypes.objects = {};
MultiPlaceableTypes.objects2 = {};
addModEventListener(MultiPlaceableTypes)

if MultiPlaceableTypes.mod == nil then
	MultiPlaceableTypes.mod = {};
end;
MultiPlaceableTypes.mod[MultiPlaceableTypes.ModDir] = true;

function MultiPlaceableTypes.placeable(xmlFilename, x,y,z, rx,ry,rz, moveMode, path,t)
	local xmlFile = loadXMLFile("TempXML", xmlFilename);
	local i=0;
	local o = {};
	if MultiPlaceableTypes.ModDir == t then
		while true do
			local key = string.format("placeable.multiPlaceableTypes.placeableType(%d)", i);
			if not hasXMLProperty(xmlFile, key) then
				break;
			end;
			local class = getXMLString(xmlFile, key .. "#class");
			local xmlFilenameObject = MultiPlaceableTypes.ModDir .. getXMLString(xmlFile, key .. "#xmlFilename");
			local classObject = PlacementScreen.placeableTypes[class];
			local placeable = classObject:new(g_currentMission:getIsServer(), g_currentMission:getIsClient());
			if placeable:load(xmlFilenameObject, x,y,z, rx,ry,rz) then
				placeable:finalizePlacement();
				--placeable:clearFoliageAndTipAreas();
				placeable:register(true);
				table.insert(o, placeable.nodeId);
				MultiPlaceableTypes.objects[placeable.nodeId] = placeable; 
				g_currentMission:removeItemToSave(placeable);
			end;	
			i = i + 1;
		end;
	end;
	delete(xmlFile);
	return o;
end;

function MultiPlaceableTypes.deleteItem(superFunc)
	if superFunc ~= nil then
		if superFunc.MultiPlaceableTypes_objectsToDelete ~= nil then
			for _, id in pairs(superFunc.MultiPlaceableTypes_objectsToDelete) do
				if MultiPlaceableTypes.objects[id] ~= nil and MultiPlaceableTypes.objects[id].delete ~= nil then
					MultiPlaceableTypes.objects[id]:unregister(true)
					MultiPlaceableTypes.objects[id]:delete();
					MultiPlaceableTypes.objects[id] = nil;
				end;		
			end;
		end;
	end;
end;	

function MultiPlaceableTypes.loadStoreItemsByXml(xmlFilename)
	local xmlFile = loadXMLFile("TempXML", xmlFilename);
	local i=0;
	while true do
		local key = string.format("placeable.multiPlaceableTypes.placeableType(%d)", i);
		if not hasXMLProperty(xmlFile, key) then
			break;
		end;
		local xmlFilenameObject = MultiPlaceableTypes.ModDir .. getXMLString(xmlFile, key .. "#xmlFilename");
		if StoreItemsUtil.storeItemsByXMLFilename[xmlFilenameObject:lower()] == nil then
			StoreItemsUtil.loadStoreItem(xmlFilenameObject, "", nil, false, false);
			local id = StoreItemsUtil.storeItemsByXMLFilename[xmlFilenameObject:lower()].id
			table.insert(MultiPlaceableTypes.storeItems, id)
		end;
		i = i + 1;
	end;
	delete(xmlFile);
end;

function MultiPlaceableTypes.loadStoreItemsByXmlName(xmlFilename)	
	if StoreItemsUtil.storeItemsByXMLFilename[xmlFilename:lower()] == nil then
		StoreItemsUtil.loadStoreItem(xmlFilename, "", nil, false, false);
		local id = StoreItemsUtil.storeItemsByXMLFilename[xmlFilename:lower()].id
		table.insert(MultiPlaceableTypes.storeItems, id)
	end;
end;

function MultiPlaceableTypes.deleteStoreItems()
	for _,id in pairs(MultiPlaceableTypes.storeItems) do
		StoreItemsUtil.removeStoreItem(id)
	end;
end;

function MultiPlaceableTypes.appendedFunctionDelete(delete)
	return function (s,...)
		MultiPlaceableTypes.deleteItem(s,...);
		delete(s,...);
	end;
end;

function MultiPlaceableTypes.appendedFunctionLoad(old, new)
	return function (xmlFilename, x,y,z, rx,ry,rz, moveMode)
		local placeable, hasNoSpace  = old(xmlFilename, x,y,z, rx,ry,rz, moveMode);
		local _, b = Utils.getModNameAndBaseDirectory(xmlFilename)
		if string.find(xmlFilename,b) then
			MultiPlaceableTypes.loadStoreItemsByXml(xmlFilename);
			--local o = new(xmlFilename, x,y,z, rx,ry,rz, b, moveMode, b);
			--if table.getn(o) > 0 then
				--placeable.MultiPlaceableTypes_objectsToDelete = o;
			--end;
			MultiPlaceableTypes.deleteStoreItems();
		end;
		return placeable, hasNoSpace;
	end;
end;

function MultiPlaceableTypes.appendedFunctionLoad2(old)
	return function (p, xmlFilename, x,y,z, rx,ry,rz, initRandom)
		MultiPlaceableTypes.loadStoreItemsByXmlName(xmlFilename);
		local item = old(p, xmlFilename, x,y,z, rx,ry,rz, initRandom);
		local _, b = Utils.getModNameAndBaseDirectory(xmlFilename)
		local o = MultiPlaceableTypes.placeable(xmlFilename, x,y,z, rx,ry,rz, b, initRandom, b)
		if table.getn(o) > 0 then
			p.MultiPlaceableTypes_objectsToDelete = o;
		end;
		MultiPlaceableTypes.deleteStoreItems();
		return item;
	end;
end;

function MultiPlaceableTypes:loadMap() end;
function MultiPlaceableTypes:mouseEvent() end;
function MultiPlaceableTypes:update() end;
function MultiPlaceableTypes:draw() end;
function MultiPlaceableTypes:keyEvent() end;

function MultiPlaceableTypes:deleteMap() end;

PlacementScreen.loadPlaceableFromXML = MultiPlaceableTypes.appendedFunctionLoad(PlacementScreen.loadPlaceableFromXML, MultiPlaceableTypes.placeable)
Placeable.load = MultiPlaceableTypes.appendedFunctionLoad2(Placeable.load)
Placeable.delete = MultiPlaceableTypes.appendedFunctionDelete(Placeable.delete)